! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! 
! verification procedure EXTautoSeeing
! 
! To run it: @@ EXTautoSeeing EFOSC_Image.fits
! Caveat: it expects a fits file with a name similar to the above
!         it knows about pix. scale of EFOSC
! ---------------------------------------------------------------
!
!.PURPOSE
!   measure the seeing on an image,
!   corrects for wavelength and airmass
!   logs the result
!.USAGE
!   @@ EXTautoSeeing image.fits
!.INPUT
!  image.fits
!.OUTPUT
!  seeing.tbl
!  seeing.log
!.HISTORY
! 100407      last modif
! 100317      kbanse   add more comparisons
! 091022      kbanse   include verification code
! 3.1 23aug00 rathreya further finetuning for EFOSC
! 3.0 05may00 msterzik adapted for EFOSC
! 2.1 22dec99 ohainaut added some safety
! 2.0 18aug99 ohainaut rebuilt after PL crash
! 1.0 ~apr99  ohainaut first PL version
!.SOME ISSUES
! -> The variable mjd cannot be defined as double - it is hardcoded in midas 
!    as a real number. as a consequence the precision of Julian date goes awry
!    after the 3rd decimal place. Subsequent digits are not truncated but given
!    junk values. Further Tcl Tk cannot pull out a double precision number from
!    midas. so the trick is to define a double variable, say meanjd, multiply
!    it by 100,000, read in TclTk as a float and then divide by 100,000. For
!    some reason TclTk is happy with this and carries the 5 decimal digits 
!    accurately after this
!-----------------------------------------------------------------------------
! 
set/midas output=yes
set/format ,f30.22
defi/par p1 ? ? "input image: "

write/out "processing {p1} for determining the image seeing ....."

!- Global
write/key filter/c/1/72 OXO
write/key arcn/c/1/72 EFOSC
write/key meanjd/d/1/1 0.
write/key lambda/r/1/1 0.
write/key logFile/c/1/72  "EFOSC_autoSeeing.log"
write/key logTable/c/1/72 "EFOSC_autoSeeing.tbl"
write/key mykey/c/1/2 " "
write/key my2key/c/1/2 " "
write/key c1/i/1/1 0
write/key c2/i/1/1 0

!- local
defi/loc line/c/1/72 oxoxo
defi/loc xfwhm/r/1/1 0.
defi/loc sxfwhm/r/1/1 0.
defi/loc yfwhm/r/1/1 0.
defi/loc syfwhm/r/1/1 0.
defi/loc ellipse/r/1/1 0.
defi/loc angle/r/1/1 0.
defi/loc sangle/r/1/1 0.
defi/loc elong/r/1/1 0.
defi/loc w/r/1/4 0. all
defi/loc i/i/1/1 0
defi/loc imscale/r/1/1 0.
defi/loc airmass/r/1/1 0.
defi/loc corXfw/r/1/1 0.
defi/loc corYfw/r/1/1 0.
defi/loc corsXf/r/1/1 0.
defi/loc corsYf/r/1/1 0.
defi/loc kappa/r/1/1 0.

!- read the image in Midas

indisk {p1} ima  >Null

!-- get the mode and scale

i = m$existd("ima.bdf","ESO.DPR.TECH")
if i .eq. 1 then
   if "{"ima.bdf",ESO.DPR.TECH}" .ne. "IMAGE" then
      writ/out "Not an image, exiting..."
      goto the_end
   endif
endif

i = m$existd("ima.bdf","ESO.INS.MOD")
if i .eq. 0 then
	i = m$existd("ima.bdf","INSTRUME")
	if i .eq. 0 then
		writ/out "*error* mode not identified..."
		return
	else
		if "{"ima.bdf",INSTRUME}" .eq. "EFOSC" then
			imscale = 0.157
		else 
		   writ/out "*warning* mode not recognized, we work in PIXELS"
		   imscale = 1.
		endif
		i = 0
	endif
!  RMA : are these EMMI modes?
else
	if "{"ima.bdf",ESO.INS.MODE}" .eq. "RILD" then
		imscale = 0.27
		
	else; if  "{"ima.bdf",ESO.INS.MODE}" .eq. "BIMG" then
		imscale = 0.37
	else
		writ/out "*warning* mode not recognized, we work in PIXELS"
		imscale = 1.
	endif; endif
endif

i = m$existd("ima.bdf","ESO.INS.FILT1.NAME")
if i .eq. 0 then
	i = m$existd("ima.bdf","ESO.INS.FILT2.NAME")
	if i .eq. 0 then
	   filter = "UNKNOWN"
	   lambda = 500.
	else
	   filter = "{ima.bdf,ESO.INS.FILT2.NAME}"
	   @@ EXTautoSeeing,lambda 
	endif
else
   filter = "{ima.bdf,ESO.INS.FILT1.NAME}"
   @@ EXTautoSeeing,lambda 
endif



!-- airmass

i = m$existd("ima.bdf","ESO.TEL.AIRM.START")
if i .gt. 0 then
   airmass = ( {ima,ESO.TEL.AIRM.START}+ {ima,ESO.TEL.AIRM.END})/2.
else
   airmass = 1.
endif


!-- search for the stars

set/cont invent | $$ cat - > /dev/null

! prepare the param. for inventory

set/midas output=no
statistics/image ima  
set/midas output=yes
 
write/key rcorr/r/1/6 31.,65535.,1113.07,669.128,76.0647,6741.33
write/key rcorr/r/8/5  1121.,159.439,256.,256.878,1186.95
@@ kcompare auto010:rcorr outputr 1,5 0.001
@@ kcompare auto011:rcorr outputr 6,6 0.005
@@ kcompare auto012:rcorr outputr 8,12 0.005
 
outputr(7) = outputr(7)-3*outputr(3)
writ/des ima lhcuts {outputr(7)},50000

wri/key INV_REAL/r/3/1 1000 
  ! in order to detect only real objects, not faint junk


search/inventory ima ima_seeing  >Null

comp/tab ima_seeing :xstart = :x - 20 
comp/tab ima_seeing :xend   = :x + 20 
comp/tab ima_seeing :ystart = :y - 20 
comp/tab ima_seeing :yend   = :Y + 20 

copy/tk -
ima_seeing :x,:y,:bg,:int,:ar,:xstart,:xend,:ystart,:yend @19 outputr/r/1/9
write/key -
rcorr 1798.0,1992.0,1079.52,3019.03,2.21028,1778.0,1818.0,1972.0,2012.0  
@@ kcompare auto020:rcorr outputr 1,9 0.05

defi/loc ww/r/1/6 0. all
ww(1) = {ima,START(1)}+100.
ww(2) = {ima,START(2)}+100.
ww(3) = {ima,START(1)}+{ima,STEP(1)}*{ima,NPIX(1)} -100.
ww(4) = {ima,START(2)}+{ima,STEP(2)}*{ima,NPIX(2)} -100.

outputi = -1
sel/tab ima_seeing.tbl :x.ge.{ww(1)} .and. :x.le.{ww(3)} .and. :y.ge.{ww(2)} .and. :y.le.{ww(4)} >Null
icorr = 17
@@ kcompare auto025:icorr outputi 1,1
 
center/gauss ima,ima_seeing ima_seeing >Null 

copy/tk -
ima_seeing :xcen,:ycen,:xerr,:yerr,:xsig,:ysig,:xfwhm,:yfwhm @10 outputr/r/1/8
write/key -
rcorr 1541.93,1125.15,1.2582e-01,1.2065e-01,5.6868,5.29922,13.3914,12.4787

@@ kcompare auto030:rcorr outputr 1,2 0.03
@@ kcompare auto031:rcorr outputr 3,6 0.0001
@@ kcompare auto032:rcorr outputr 7,8 0.03

!-- measure the seeing

comp/tab ima_seeing :xfwhms = :xfwhm * {imscale}
comp/tab ima_seeing :yfwhms = :yfwhm * {imscale}
comp/tab ima_seeing :ellipse = :xfwhm / :yfwhm

sele/tab ima_seeing :status .eq. 0 .and. :xfwhms .ge. .2 .and. :yfwhms .ge. .2  .and. :icent .le. 40000 >Null
icorr = 14
@@ kcompare auto040:icorr outputi 1,1

sele/tab -
ima_seeing sel.eq.1 .and.  :ellipse .le. 1.25 .and. :ellipse .ge. 0.8 >Null
icorr = 7
@@ kcompare auto044:icorr outputi 1,1

stat/tab ima_seeing :xfwhm  >Null
write/keyw rcorr 8.84334,13.3914,10.0244,1.65788
@@ kcompare auto047:rcorr outputr 1,4 0.0005

xfwhm = {outputr(3)}
sxfwhm = outputr(4)

w(1) = xfwhm - 2* sxfwhm
w(2) = xfwhm + 2* sxfwhm

stat/tab ima_seeing :yfwhm   >Null
write/keyw rcorr 7.10707,12.4787,8.48048,1.86096  
@@ kcompare auto048:rcorr outputr 1,4 0.0005

yfwhm = outputr(3)
syfwhm = outputr(4)

kappa = 2.

again:
w(3) = yfwhm - kappa* syfwhm
w(4) = yfwhm + kappa* syfwhm

sele/tab ima_seeing sel.eq.1 .and. :status.eq.0 .and.:xfwhm.ge.{w(1)}.and.:xfwhm.le.{w(2)}  >Null
icorr = 6
@@ kcompare auto050:icorr outputi 1,1

sele/tab ima_seeing sel.eq.1 .and.:yfwhm.ge.{w(3)}.and.:yfwhm.le.{w(4)}  >Null
icorr = 6
@@ kcompare auto053:icorr outputi 1,1

if outputi(1) .eq. 0 then
! all stars were delete...
  if kappa .gt. 3 then
	writ/out "could not find stars... exiting"
	goto the_end
  endif
  kappa = 4
  goto again
endif

stat/tab ima_seeing :xfwhm  >Null
write/keyw rcorr  8.84334,11.0421,9.46326,0.808161 
@@ kcompare auto060:rcorr outputr 1,4 0.0005

w(1) = outputr(3) 
w(2) = outputr(4)

stat/tab ima_seeing :yfwhm  >Null
write/keyw rcorr   7.10707,9.05224,7.81411,0.65251 
@@ kcompare auto064:rcorr outputr 1,4 0.0005

w(3) = outputr(3)
w(4) = outputr(4)

! stat/tab ima_seeing :angle | $$ cat - > /dev/null
! angle = outputr(3)
! sangle = outputr(4)

elong = xfwhm/yfwhm

xfwhm = w(1) * imscale
sxfwhm = w(2) * imscale
yfwhm = w(3) * imscale
syfwhm = w(4) * imscale



!cle/ch ov
!load/tab ima_seeing :xcen :ycen ? ? 4 4 4 

!-- compute corrected seeing (Zenith, 500nm)
corXfw = {xfwhm} * ({lambda}/500.)**(.2) * {airmass}**(-.6)
corYfw = {yfwhm} * ({lambda}/500.)**(.2) * {airmass}**(-.6)
corsXf = {sxfwhm} * ({lambda}/500.)**(.2) * {airmass}**(-.6)
corsYf = {syfwhm} * ({lambda}/500.)**(.2) * {airmass}**(-.6)

!-- print out the result

meanjd = {ima.bdf,O_Time(4)} * 100000
comp/keyw outputd = m$abs(meanjd - 5493202574.5540008)
outputr = outputd
rcorr = 0.0
@@ kcompare auto070:rcorr outputr 1,1 0.000001
 
set/form f8.3
writ/out "{p1}"
writ/out "RA={{p1},o_pos(1)}  DEC={{p1},o_pos(2)}  AIRMASS={airmass}  FILTER={filter}"
write/keyw rcorr {{p1},o_pos(1)},{{p1},o_pos(2)}
rcorr(3) = airmass
write/keyw rrefdat 92.727,-44.996,1.281 
@@ kcompare auto077:rrefdat rcorr 1,3 0.001
 
writ/out "SEEING :"
writ/out "RAW         {xfwhm} ({sxfwhm})   x   {yfwhm} ({syfwhm})"
writ/out "NORMALISED  {corXfw} ({corsXf})   x   {corYfw} ({corsYf})   (AIRMASS=1, LAMBDA=5000A)" 

rcorr(1) = xfwhm
rcorr(2) = sxfwhm
rcorr(3) = yfwhm
rcorr(4) = syfwhm
rcorr(5) = corXfw
rcorr(6) = corsXf
rcorr(7) = corYfw
rcorr(8) = corsYf
write/keyw rrefdat 1.4857,0.1269,1.2268,0.1024,1.3464,0.1149,1.1117,0.0928
@@ kcompare auto079:rrefdat rcorr 1,8 0.0001

outputr(1) = {airmass}
outputr(2) = {xfwhm}
outputr(3) = {yfwhm}
outputr(4) = {corXfw}
outputr(5) = {corYfw}
outputr(6) = {corsXf}
outputr(7) = {corsYf}
outputd(1) = {meanjd}


!--- END
the_end:

!----------------------------------------------------------------------------
!  RMA : change these to EFOSC values
entry lambda
!!write/out "entry LAMBDA: lmbda = {lambda} at start"

lambda = 499.
if "{filter(1:1)}" .eq. "U"   lambda = 357.078
if "{filter(1:1)}" .eq. "B"   lambda = 421.202
if "{filter(1:1)}" .eq. "V"   lambda = 544.170
if "{filter(1:1)}" .eq. "R"   lambda = 641.584
if "{filter(1:1)}" .eq. "i"   lambda = 794.960
if "{filter(1:1)}" .eq. "z"   lambda = 840.903

!!write/out "entry LAMBDA: lmbda = {lambda} at end"
return
!------------------------------------------------------------------------------
